program ResetHubs;

uses
	WinAPI.Windows,
	System.SysUtils, WinAPI.ShellAPI,
	SetupDi, UACBridge;

{$R *.res}

const
	HubClassGUID: TGUID = (D1: $F18A0E88; D2: $C30C; D3: $11D0; D4: ($88, $15, $00, $A0, $C9, $06, $BE, $D8));

	FILE_DEVICE_UNKNOWN = $00000022;
	FILE_DEVICE_USB = FILE_DEVICE_UNKNOWN;

{
const
	USB_HUB_CYCLE_PORT = $111;
	IOCTL_USB_HUB_CYCLE_PORT = ((FILE_DEVICE_USB) shl 16) or ((FILE_ANY_ACCESS) shl 14) or ((USB_HUB_CYCLE_PORT) shl 2) or (METHOD_BUFFERED);
	//CTL_CODE(FILE_DEVICE_USB, USB_HUB_CYCLE_PORT, METHOD_BUFFERED, FILE_ANY_ACCESS)
type
	TUSBCyclePortParams = packed record
		// 1-based port number
		ConnectionIndex: LongWord; // INPUT

		StatusReturned: LongWord; // OUTPUT
	end;
	TPUSBCyclePortParams = ^TUSBCyclePortParams;
	USB_CYCLE_PORT_PARAMS = TUSBCyclePortParams;
	PUSB_CYCLE_PORT_PARAMS = TPUSBCyclePortParams;

function USBCyclePort(hDevice: THandle; iPort: LongWord): LongWord;
var
	UCPP: TUSBCyclePortParams;
	BytesReturned: LongWord;

begin
	UCPP.ConnectionIndex := iPort;
	UCPP.StatusReturned := 0;
	if DeviceIoControl(hDevice, IOCTL_USB_HUB_CYCLE_PORT, @UCPP, SizeOf(UCPP), @UCPP, SizeOf(UCPP), BytesReturned, nil) then
		Result := ERROR_SUCCESS
	else
		Result := GetLastError
	;
end;
}

const
	USB_RESET_HUB = $113;
	IOCTL_USB_RESET_HUB = ((FILE_DEVICE_USB) shl 16) or ((FILE_ANY_ACCESS) shl 14) or ((USB_RESET_HUB) shl 2) or (METHOD_BUFFERED);
	//CTL_CODE(FILE_DEVICE_USB, USB_RESET_HUB, METHOD_BUFFERED, FILE_ANY_ACCESS)

function USBResetHub(hDevice: THandle): LongWord;
var
	BytesReturned: LongWord;

begin
	if DeviceIoControl(hDevice, IOCTL_USB_RESET_HUB, nil, 0, nil, 0, BytesReturned, nil) then
		Result := ERROR_SUCCESS
	else
		Result := GetLastError
	;
end;

var
	DevicePaths: TStringArray;
	Buf, ThisPath: WideString;
	H, I: Integer;
	hDevice: THandle;
	Status: LongWord;

begin
	if not IsAdminLevel then begin
		ThisPath := ParamStr(0);
		if ShellExecute(0, 'runas', PWideChar(ThisPath), '', '.', SW_SHOWNORMAL) > 32 then
			  Status := ERROR_SUCCESS
		else
			  Status := GetLastError
		;
		ThisPath := '';

		if Status <> ERROR_CANCELLED then
			  Exit
		;
	end;

	DevicePaths := GetDevicePaths(HubClassGUID);

	H := High(DevicePaths);
	for I := H downto 0 do begin
		if Pos('#VID_1605&PID_F001#', UpperCase(DevicePaths[I])) <> 0 then Continue;

		if I < H then DevicePaths[I] := DevicePaths[H];
		SetLength(DevicePaths, H);
		Dec(H);
	end;

	{
	Buf := IntToStr(Length(DevicePaths));
	for I := 0 to High(DevicePaths) do
		Buf := Buf + (#13#10 + DevicePaths[I])
	;
	MessageBox(0, @Buf[1], 'Our DevicePaths', MB_ICONINFORMATION);
	Buf := '';
	}

	for I := 0 to High(DevicePaths) do begin
		ThisPath := DevicePaths[I];
		hDevice := CreateFileW(PWideChar(ThisPath), GENERIC_WRITE or GENERIC_READ, FILE_SHARE_WRITE or FILE_SHARE_READ, nil, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);
		ThisPath := '';

		Status := USBResetHub(hDevice);

		CloseHandle(hDevice);

		if Status <> ERROR_SUCCESS then begin
			Buf := 'Error ' + IntToStr(Status) + ' resetting hub. DevicePath = ' + DevicePaths[I];
			MessageBox(0, @Buf[1], 'ResetHubs Error', MB_ICONEXCLAMATION);
			Buf := '';
		end;
	end;
end.
